@@ -1,5 +1,5 @@
 import pytest
-from datetime import datetime
+from datetime import datetime, timezone
 from uuid import UUID
 
 from deepdiff import DeepDiff
@@ -64,7 +64,7 @@ mock_node_uuid = str(uuid4_from_hash(MockNode.__qualname__))
         (
             WorkflowExecutionInitiatedEvent(
                 id=UUID("123e4567-e89b-12d3-a456-426614174000"),
-                timestamp=datetime(2024, 1, 1, 12, 0, 0),
+                timestamp=datetime(2024, 1, 1, 12, 0, 0, tzinfo=timezone.utc),
                 trace_id=UUID("123e4567-e89b-12d3-a456-426614174000"),
                 span_id=UUID("123e4567-e89b-12d3-a456-426614174000"),
                 body=WorkflowExecutionInitiatedBody(
@@ -75,7 +75,7 @@ mock_node_uuid = str(uuid4_from_hash(MockNode.__qualname__))
             {
                 "id": "123e4567-e89b-12d3-a456-426614174000",
                 "api_version": "2024-10-25",
-                "timestamp": "2024-01-01T12:00:00",
+                "timestamp": "2024-01-01T12:00:00Z",
                 "trace_id": "123e4567-e89b-12d3-a456-426614174000",
                 "span_id": "123e4567-e89b-12d3-a456-426614174000",
                 "name": "workflow.execution.initiated",
@@ -90,14 +90,16 @@ mock_node_uuid = str(uuid4_from_hash(MockNode.__qualname__))
                     },
                     "display_context": None,
                     "initial_state": None,
+                    "workflow_version_exec_config": None,
                 },
                 "parent": None,
+                "links": None,
             },
         ),
         (
             NodeExecutionInitiatedEvent(
                 id=UUID("123e4567-e89b-12d3-a456-426614174000"),
-                timestamp=datetime(2024, 1, 1, 12, 0, 0),
+                timestamp=datetime(2024, 1, 1, 12, 0, 0, tzinfo=timezone.utc),
                 trace_id=UUID("123e4567-e89b-12d3-a456-426614174000"),
                 span_id=UUID("123e4567-e89b-12d3-a456-426614174000"),
                 body=NodeExecutionInitiatedBody(
@@ -119,7 +121,7 @@ mock_node_uuid = str(uuid4_from_hash(MockNode.__qualname__))
             {
                 "id": "123e4567-e89b-12d3-a456-426614174000",
                 "api_version": "2024-10-25",
-                "timestamp": "2024-01-01T12:00:00",
+                "timestamp": "2024-01-01T12:00:00Z",
                 "trace_id": "123e4567-e89b-12d3-a456-426614174000",
                 "span_id": "123e4567-e89b-12d3-a456-426614174000",
                 "name": "node.execution.initiated",
@@ -155,12 +157,13 @@ mock_node_uuid = str(uuid4_from_hash(MockNode.__qualname__))
                     "type": "WORKFLOW_NODE",
                     "span_id": "123e4567-e89b-12d3-a456-426614174000",
                 },
+                "links": None,
             },
         ),
         (
             WorkflowExecutionStreamingEvent(
                 id=UUID("123e4567-e89b-12d3-a456-426614174000"),
-                timestamp=datetime(2024, 1, 1, 12, 0, 0),
+                timestamp=datetime(2024, 1, 1, 12, 0, 0, tzinfo=timezone.utc),
                 trace_id=UUID("123e4567-e89b-12d3-a456-426614174000"),
                 span_id=UUID("123e4567-e89b-12d3-a456-426614174000"),
                 body=WorkflowExecutionStreamingBody(
@@ -174,7 +177,7 @@ mock_node_uuid = str(uuid4_from_hash(MockNode.__qualname__))
             {
                 "id": "123e4567-e89b-12d3-a456-426614174000",
                 "api_version": "2024-10-25",
-                "timestamp": "2024-01-01T12:00:00",
+                "timestamp": "2024-01-01T12:00:00Z",
                 "trace_id": "123e4567-e89b-12d3-a456-426614174000",
                 "span_id": "123e4567-e89b-12d3-a456-426614174000",
                 "name": "workflow.execution.streaming",
@@ -190,12 +193,13 @@ mock_node_uuid = str(uuid4_from_hash(MockNode.__qualname__))
                     },
                 },
                 "parent": None,
+                "links": None,
             },
         ),
         (
             WorkflowExecutionFulfilledEvent(
                 id=UUID("123e4567-e89b-12d3-a456-426614174000"),
-                timestamp=datetime(2024, 1, 1, 12, 0, 0),
+                timestamp=datetime(2024, 1, 1, 12, 0, 0, tzinfo=timezone.utc),
                 trace_id=UUID("123e4567-e89b-12d3-a456-426614174000"),
                 span_id=UUID("123e4567-e89b-12d3-a456-426614174000"),
                 body=WorkflowExecutionFulfilledBody(
@@ -208,7 +212,7 @@ mock_node_uuid = str(uuid4_from_hash(MockNode.__qualname__))
             {
                 "id": "123e4567-e89b-12d3-a456-426614174000",
                 "api_version": "2024-10-25",
-                "timestamp": "2024-01-01T12:00:00",
+                "timestamp": "2024-01-01T12:00:00Z",
                 "trace_id": "123e4567-e89b-12d3-a456-426614174000",
                 "span_id": "123e4567-e89b-12d3-a456-426614174000",
                 "name": "workflow.execution.fulfilled",
@@ -223,12 +227,13 @@ mock_node_uuid = str(uuid4_from_hash(MockNode.__qualname__))
                     },
                 },
                 "parent": None,
+                "links": None,
             },
         ),
         (
             WorkflowExecutionRejectedEvent(
                 id=UUID("123e4567-e89b-12d3-a456-426614174000"),
-                timestamp=datetime(2024, 1, 1, 12, 0, 0),
+                timestamp=datetime(2024, 1, 1, 12, 0, 0, tzinfo=timezone.utc),
                 trace_id=UUID("123e4567-e89b-12d3-a456-426614174000"),
                 span_id=UUID("123e4567-e89b-12d3-a456-426614174000"),
                 body=WorkflowExecutionRejectedBody(
@@ -242,7 +247,7 @@ mock_node_uuid = str(uuid4_from_hash(MockNode.__qualname__))
             {
                 "id": "123e4567-e89b-12d3-a456-426614174000",
                 "api_version": "2024-10-25",
-                "timestamp": "2024-01-01T12:00:00",
+                "timestamp": "2024-01-01T12:00:00Z",
                 "trace_id": "123e4567-e89b-12d3-a456-426614174000",
                 "span_id": "123e4567-e89b-12d3-a456-426614174000",
                 "name": "workflow.execution.rejected",
@@ -258,12 +263,13 @@ mock_node_uuid = str(uuid4_from_hash(MockNode.__qualname__))
                     },
                 },
                 "parent": None,
+                "links": None,
             },
         ),
         (
             NodeExecutionStreamingEvent(
                 id=UUID("123e4567-e89b-12d3-a456-426614174000"),
-                timestamp=datetime(2024, 1, 1, 12, 0, 0),
+                timestamp=datetime(2024, 1, 1, 12, 0, 0, tzinfo=timezone.utc),
                 trace_id=UUID("123e4567-e89b-12d3-a456-426614174000"),
                 span_id=UUID("123e4567-e89b-12d3-a456-426614174000"),
                 body=NodeExecutionStreamingBody(
@@ -277,7 +283,7 @@ mock_node_uuid = str(uuid4_from_hash(MockNode.__qualname__))
             {
                 "id": "123e4567-e89b-12d3-a456-426614174000",
                 "api_version": "2024-10-25",
-                "timestamp": "2024-01-01T12:00:00",
+                "timestamp": "2024-01-01T12:00:00Z",
                 "trace_id": "123e4567-e89b-12d3-a456-426614174000",
                 "span_id": "123e4567-e89b-12d3-a456-426614174000",
                 "name": "node.execution.streaming",
@@ -293,12 +299,13 @@ mock_node_uuid = str(uuid4_from_hash(MockNode.__qualname__))
                     },
                 },
                 "parent": None,
+                "links": None,
             },
         ),
         (
             NodeExecutionFulfilledEvent(
                 id=UUID("123e4567-e89b-12d3-a456-426614174000"),
-                timestamp=datetime(2024, 1, 1, 12, 0, 0),
+                timestamp=datetime(2024, 1, 1, 12, 0, 0, tzinfo=timezone.utc),
                 trace_id=UUID("123e4567-e89b-12d3-a456-426614174000"),
                 span_id=UUID("123e4567-e89b-12d3-a456-426614174000"),
                 body=NodeExecutionFulfilledBody(
@@ -312,7 +319,7 @@ mock_node_uuid = str(uuid4_from_hash(MockNode.__qualname__))
             {
                 "id": "123e4567-e89b-12d3-a456-426614174000",
                 "api_version": "2024-10-25",
-                "timestamp": "2024-01-01T12:00:00",
+                "timestamp": "2024-01-01T12:00:00Z",
                 "trace_id": "123e4567-e89b-12d3-a456-426614174000",
                 "span_id": "123e4567-e89b-12d3-a456-426614174000",
                 "name": "node.execution.fulfilled",
@@ -333,12 +340,13 @@ mock_node_uuid = str(uuid4_from_hash(MockNode.__qualname__))
                     "mocked": None,
                 },
                 "parent": None,
+                "links": None,
             },
         ),
         (
             NodeExecutionFulfilledEvent(
                 id=UUID("123e4567-e89b-12d3-a456-426614174000"),
-                timestamp=datetime(2024, 1, 1, 12, 0, 0),
+                timestamp=datetime(2024, 1, 1, 12, 0, 0, tzinfo=timezone.utc),
                 trace_id=UUID("123e4567-e89b-12d3-a456-426614174000"),
                 span_id=UUID("123e4567-e89b-12d3-a456-426614174000"),
                 body=NodeExecutionFulfilledBody(
@@ -352,7 +360,7 @@ mock_node_uuid = str(uuid4_from_hash(MockNode.__qualname__))
             {
                 "id": "123e4567-e89b-12d3-a456-426614174000",
                 "api_version": "2024-10-25",
-                "timestamp": "2024-01-01T12:00:00",
+                "timestamp": "2024-01-01T12:00:00Z",
                 "trace_id": "123e4567-e89b-12d3-a456-426614174000",
                 "span_id": "123e4567-e89b-12d3-a456-426614174000",
                 "name": "node.execution.fulfilled",
@@ -371,12 +379,13 @@ mock_node_uuid = str(uuid4_from_hash(MockNode.__qualname__))
                     "mocked": None,
                 },
                 "parent": None,
+                "links": None,
             },
         ),
         (
             NodeExecutionFulfilledEvent(
                 id=UUID("123e4567-e89b-12d3-a456-426614174000"),
-                timestamp=datetime(2024, 1, 1, 12, 0, 0),
+                timestamp=datetime(2024, 1, 1, 12, 0, 0, tzinfo=timezone.utc),
                 trace_id=UUID("123e4567-e89b-12d3-a456-426614174000"),
                 span_id=UUID("123e4567-e89b-12d3-a456-426614174000"),
                 body=NodeExecutionFulfilledBody(
@@ -390,7 +399,7 @@ mock_node_uuid = str(uuid4_from_hash(MockNode.__qualname__))
             {
                 "id": "123e4567-e89b-12d3-a456-426614174000",
                 "api_version": "2024-10-25",
-                "timestamp": "2024-01-01T12:00:00",
+                "timestamp": "2024-01-01T12:00:00Z",
                 "trace_id": "123e4567-e89b-12d3-a456-426614174000",
                 "span_id": "123e4567-e89b-12d3-a456-426614174000",
                 "name": "node.execution.fulfilled",
@@ -404,6 +413,7 @@ mock_node_uuid = str(uuid4_from_hash(MockNode.__qualname__))
                     "mocked": True,
                 },
                 "parent": None,
+                "links": None,
             },
         ),
     ],
