@@ -5,6 +5,8 @@ from typing import Any, Callable, Dict, Iterator, List, Optional, Type, Union, c
 from pydash import snake_case
 
 from vellum import ChatMessage, PromptBlock
+from vellum.client.types.array_chat_message_content import ArrayChatMessageContent
+from vellum.client.types.array_chat_message_content_item import ArrayChatMessageContentItem
 from vellum.client.types.function_call_chat_message_content import FunctionCallChatMessageContent
 from vellum.client.types.function_call_chat_message_content_value import FunctionCallChatMessageContentValue
 from vellum.client.types.function_definition import FunctionDefinition
@@ -17,6 +19,7 @@ from vellum.workflows.exceptions import NodeException
 from vellum.workflows.expressions.concat import ConcatExpression
 from vellum.workflows.inputs import BaseInputs
 from vellum.workflows.integrations.composio_service import ComposioService
+from vellum.workflows.integrations.mcp_service import MCPService
 from vellum.workflows.nodes.bases import BaseNode
 from vellum.workflows.nodes.core.inline_subworkflow_node.node import InlineSubworkflowNode
 from vellum.workflows.nodes.displayable.inline_prompt_node.node import InlinePromptNode
@@ -27,8 +30,8 @@ from vellum.workflows.ports.port import Port
 from vellum.workflows.references.lazy import LazyReference
 from vellum.workflows.state import BaseState
 from vellum.workflows.state.encoder import DefaultStateEncoder
-from vellum.workflows.types.core import EntityInputsInterface, MergeBehavior, Tool
-from vellum.workflows.types.definition import ComposioToolDefinition, DeploymentDefinition
+from vellum.workflows.types.core import EntityInputsInterface, MergeBehavior, Tool, ToolBase
+from vellum.workflows.types.definition import ComposioToolDefinition, DeploymentDefinition, MCPServer, MCPToolDefinition
 from vellum.workflows.types.generics import is_workflow_class
 
 CHAT_HISTORY_VARIABLE = "chat_history"
@@ -44,8 +47,9 @@ class FunctionCallNodeMixin:
 
     def _extract_function_arguments(self) -> dict:
         """Extract arguments from function call output."""
-        if self.function_call_output and len(self.function_call_output) > 0:
-            function_call = self.function_call_output[0]
+        current_index = getattr(self, "state").current_prompt_output_index
+        if self.function_call_output and len(self.function_call_output) > current_index:
+            function_call = self.function_call_output[current_index]
             if function_call.type == "FUNCTION_CALL" and function_call.value is not None:
                 return function_call.value.arguments or {}
         return {}
@@ -58,9 +62,12 @@ class FunctionCallNodeMixin:
                 content=StringChatMessageContent(value=json.dumps(result, cls=DefaultStateEncoder)),
             )
         )
+        with state.__quiet__():
+            state.current_function_calls_processed += 1
+            state.current_prompt_output_index += 1
 
 
-class ToolRouterNode(InlinePromptNode[ToolCallingState]):
+class ToolPromptNode(InlinePromptNode[ToolCallingState]):
     max_prompt_iterations: Optional[int] = 5
 
     class Trigger(InlinePromptNode.Trigger):
@@ -72,32 +79,46 @@ class ToolRouterNode(InlinePromptNode[ToolCallingState]):
             raise NodeException(message=max_iterations_message, code=WorkflowErrorCode.NODE_EXECUTION)
 
         generator = super().run()
+        with self.state.__quiet__():
+            self.state.current_prompt_output_index = 0
+            self.state.current_function_calls_processed = 0
+            self.state.prompt_iterations += 1
         for output in generator:
-            if output.name == "results" and output.value:
-                values = cast(List[Any], output.value)
-                if values and len(values) > 0:
-                    if values[0].type == "STRING":
-                        self.state.chat_history.append(ChatMessage(role="ASSISTANT", text=values[0].value))
-                    elif values[0].type == "FUNCTION_CALL":
-                        self.state.prompt_iterations += 1
-
-                        function_call = values[0].value
-                        if function_call is not None:
-                            self.state.chat_history.append(
-                                ChatMessage(
-                                    role="ASSISTANT",
-                                    content=FunctionCallChatMessageContent(
-                                        value=FunctionCallChatMessageContentValue(
-                                            name=function_call.name,
-                                            arguments=function_call.arguments,
-                                            id=function_call.id,
-                                        ),
-                                    ),
-                                )
+            if output.name == InlinePromptNode.Outputs.results.name and output.value:
+                prompt_outputs = cast(List[PromptOutput], output.value)
+                chat_contents: List[ArrayChatMessageContentItem] = []
+                for prompt_output in prompt_outputs:
+                    if prompt_output.type == "STRING":
+                        chat_contents.append(StringChatMessageContent(value=prompt_output.value))
+                    elif prompt_output.type == "FUNCTION_CALL" and prompt_output.value:
+                        raw_function_call = prompt_output.value.model_dump()
+                        if "state" in raw_function_call:
+                            del raw_function_call["state"]
+                        chat_contents.append(
+                            FunctionCallChatMessageContent(
+                                value=FunctionCallChatMessageContentValue.model_validate(raw_function_call)
                             )
+                        )
+
+                if len(chat_contents) == 1:
+                    if chat_contents[0].type == "STRING":
+                        self.state.chat_history.append(ChatMessage(role="ASSISTANT", text=chat_contents[0].value))
+                    else:
+                        self.state.chat_history.append(ChatMessage(role="ASSISTANT", content=chat_contents[0]))
+                else:
+                    self.state.chat_history.append(
+                        ChatMessage(role="ASSISTANT", content=ArrayChatMessageContent(value=chat_contents))
+                    )
+
             yield output
 
 
+class RouterNode(BaseNode[ToolCallingState]):
+    """Router node that handles routing to function nodes based on outputs."""
+
+    pass
+
+
 class DynamicSubworkflowDeploymentNode(SubworkflowDeploymentNode[ToolCallingState], FunctionCallNodeMixin):
     """Node that executes a deployment definition with function call output."""
 
@@ -108,8 +129,8 @@ class DynamicSubworkflowDeploymentNode(SubworkflowDeploymentNode[ToolCallingStat
         # we do in the `__init__` method. Long term, instead of the function_call_output attribute above, we
         # want to do:
         # ```python
-        # subworkflow_inputs = tool_router_node.Outputs.results[0]['value']['arguments'].if_(
-        #     tool_router_node.Outputs.results[0]['type'].equals('FUNCTION_CALL'),
+        # subworkflow_inputs = tool_prompt_node.Outputs.results[0]['value']['arguments'].if_(
+        #     tool_prompt_node.Outputs.results[0]['type'].equals('FUNCTION_CALL'),
         #     {},
         # )
         # ```
@@ -183,7 +204,12 @@ class ComposioNode(BaseNode[ToolCallingState], FunctionCallNodeMixin):
         try:
             # Execute using ComposioService
             composio_service = ComposioService()
-            result = composio_service.execute_tool(tool_name=self.composio_tool.action, arguments=arguments)
+            if self.composio_tool.user_id is not None:
+                result = composio_service.execute_tool(
+                    tool_name=self.composio_tool.action, arguments=arguments, user_id=self.composio_tool.user_id
+                )
+            else:
+                result = composio_service.execute_tool(tool_name=self.composio_tool.action, arguments=arguments)
         except Exception as e:
             raise NodeException(
                 message=f"Error executing Composio tool '{self.composio_tool.action}': {str(e)}",
@@ -196,93 +222,126 @@ class ComposioNode(BaseNode[ToolCallingState], FunctionCallNodeMixin):
         yield from []
 
 
-def _hydrate_composio_tool_definition(tool_def: ComposioToolDefinition) -> ComposioToolDefinition:
+class MCPNode(BaseNode[ToolCallingState], FunctionCallNodeMixin):
+    """Node that executes an MCP tool with function call output."""
+
+    mcp_tool: MCPToolDefinition
+
+    def run(self) -> Iterator[BaseOutput]:
+        arguments = self._extract_function_arguments()
+
+        try:
+            mcp_service = MCPService()
+            result = mcp_service.execute_tool(tool_def=self.mcp_tool, arguments=arguments)
+        except Exception as e:
+            raise NodeException(
+                message=f"Error executing MCP tool '{self.mcp_tool.name}': {str(e)}",
+                code=WorkflowErrorCode.NODE_EXECUTION,
+            )
+
+        # Add result to chat history
+        self._add_function_result_to_chat_history(result, self.state)
+
+        yield from []
+
+
+class ElseNode(BaseNode[ToolCallingState]):
+    """Node that executes when no function conditions match."""
+
+    class Ports(BaseNode.Ports):
+        # Redefined in the create_else_node function, but defined here to resolve mypy errors
+        loop_to_router = Port.on_if(
+            ToolCallingState.current_prompt_output_index.less_than(ToolPromptNode.Outputs.results.length())
+        )
+        loop_to_prompt = Port.on_elif(ToolCallingState.current_function_calls_processed.greater_than(0))
+        end = Port.on_else()
+
+    def run(self) -> BaseNode.Outputs:
+        with self.state.__quiet__():
+            self.state.current_prompt_output_index += 1
+        return self.Outputs()
+
+
+def _hydrate_composio_tool_definition(tool_def: ComposioToolDefinition) -> FunctionDefinition:
     """Hydrate a ComposioToolDefinition with detailed information from the Composio API.
 
     Args:
         tool_def: The basic ComposioToolDefinition to enhance
 
     Returns:
-        ComposioToolDefinition with detailed parameters and description
+        FunctionDefinition with detailed parameters and description
     """
     try:
         composio_service = ComposioService()
         tool_details = composio_service.get_tool_by_slug(tool_def.action)
 
-        # Extract toolkit information from API response
-        toolkit_info = tool_details.get("toolkit", {})
-        toolkit_slug = (
-            toolkit_info.get("slug", tool_def.toolkit) if isinstance(toolkit_info, dict) else tool_def.toolkit
-        )
-
-        # Create a version of the tool definition with proper field extraction
-        return ComposioToolDefinition(
-            type=tool_def.type,
-            toolkit=toolkit_slug.upper() if toolkit_slug else tool_def.toolkit,
-            action=tool_details.get("slug", tool_def.action),
+        # Create a FunctionDefinition directly with proper field extraction
+        return FunctionDefinition(
+            name=tool_def.name,
             description=tool_details.get("description", tool_def.description),
-            display_name=tool_details.get("name", tool_def.display_name),
-            parameters=tool_details.get("input_parameters", tool_def.parameters),
-            version=tool_details.get("version", tool_def.version),
-            tags=tool_details.get("tags", tool_def.tags),
+            parameters=tool_details.get("input_parameters", {}),
         )
 
     except Exception as e:
-        # If hydration fails (including no API key), log and return original
+        # If hydration fails (including no API key), log and return basic function definition
         logger.warning(f"Failed to enhance Composio tool '{tool_def.action}': {e}")
-        return tool_def
+        return FunctionDefinition(
+            name=tool_def.name,
+            description=tool_def.description,
+            parameters={},
+        )
 
 
-def create_tool_router_node(
+def hydrate_mcp_tool_definitions(server_def: MCPServer) -> List[MCPToolDefinition]:
+    """Hydrate an MCPToolDefinition with detailed information from the MCP server.
+
+    We do tool discovery on the MCP server to get the tool definitions.
+
+    Args:
+        tool_def: The basic MCPToolDefinition to enhance
+
+    Returns:
+        MCPToolDefinition with detailed parameters and description
+    """
+    try:
+        mcp_service = MCPService()
+        return mcp_service.hydrate_tool_definitions(server_def)
+    except Exception as e:
+        # If hydration fails, log and return original
+        logger.warning(f"Failed to enhance MCP server '{server_def.name}': {e}")
+        return []
+
+
+def create_tool_prompt_node(
     ml_model: str,
     blocks: List[Union[PromptBlock, Dict[str, Any]]],
     functions: List[Tool],
     prompt_inputs: Optional[EntityInputsInterface],
     parameters: PromptParameters,
     max_prompt_iterations: Optional[int] = None,
-) -> Type[ToolRouterNode]:
+) -> Type[ToolPromptNode]:
     if functions and len(functions) > 0:
-        # Create dynamic ports and convert functions in a single loop
-        Ports = type("Ports", (), {})
         prompt_functions: List[Union[Tool, FunctionDefinition]] = []
 
         for function in functions:
             if isinstance(function, ComposioToolDefinition):
                 # Get Composio tool details and hydrate the function definition
                 enhanced_function = _hydrate_composio_tool_definition(function)
-                prompt_functions.append(
-                    FunctionDefinition(
-                        name=enhanced_function.name,
-                        description=enhanced_function.description,
-                        parameters=enhanced_function.parameters,
+                prompt_functions.append(enhanced_function)
+            elif isinstance(function, MCPServer):
+                tool_functions: List[MCPToolDefinition] = hydrate_mcp_tool_definitions(function)
+                for tool_function in tool_functions:
+                    name = get_mcp_tool_name(tool_function)
+                    prompt_functions.append(
+                        FunctionDefinition(
+                            name=name,
+                            description=tool_function.description,
+                            parameters=tool_function.parameters,
+                        )
                     )
-                )
             else:
                 prompt_functions.append(function)
-
-            # Create port for this function (using original function for get_function_name)
-            function_name = get_function_name(function)
-
-            # Avoid using lambda to capture function_name
-            # lambda will capture the function_name by reference,
-            # and if the function_name is changed, the port_condition will also change.
-            def create_port_condition(fn_name):
-                return LazyReference(
-                    lambda: (
-                        node.Outputs.results[0]["type"].equals("FUNCTION_CALL")
-                        & node.Outputs.results[0]["value"]["name"].equals(fn_name)
-                    )
-                )
-
-            port_condition = create_port_condition(function_name)
-            port = Port.on_if(port_condition)
-            setattr(Ports, function_name, port)
-
-        # Add the else port for when no function conditions match
-        setattr(Ports, "default", Port.on_else())
     else:
-        # If no functions exist, create a simple Ports class with just a default port
-        Ports = type("Ports", (), {"default": Port(default=True)})
         prompt_functions = []
 
     # Add a chat history block to blocks only if one doesn't already exist
@@ -318,10 +377,10 @@ def create_tool_router_node(
     }
 
     node = cast(
-        Type[ToolRouterNode],
+        Type[ToolPromptNode],
         type(
-            "ToolRouterNode",
-            (ToolRouterNode,),
+            "ToolPromptNode",
+            (ToolPromptNode,),
             {
                 "ml_model": ml_model,
                 "blocks": blocks,
@@ -329,7 +388,73 @@ def create_tool_router_node(
                 "prompt_inputs": node_prompt_inputs,
                 "parameters": parameters,
                 "max_prompt_iterations": max_prompt_iterations,
+                "__module__": __name__,
+            },
+        ),
+    )
+    return node
+
+
+def create_router_node(
+    functions: List[Tool],
+    tool_prompt_node: Type[InlinePromptNode[ToolCallingState]],
+) -> Type[RouterNode]:
+    """Create a RouterNode with dynamic ports that route based on tool_prompt_node outputs."""
+
+    if functions and len(functions) > 0:
+        # Create dynamic ports and convert functions in a single loop
+        Ports = type("Ports", (), {})
+
+        # Avoid using lambda to capture function_name
+        # lambda will capture the function_name by reference,
+        # and if the function_name is changed, the port_condition will also change.
+        def create_port_condition(fn_name):
+            return Port.on_if(
+                LazyReference(
+                    lambda: (
+                        ToolCallingState.current_prompt_output_index.less_than(
+                            tool_prompt_node.Outputs.results.length()
+                        )
+                        & tool_prompt_node.Outputs.results[ToolCallingState.current_prompt_output_index]["type"].equals(
+                            "FUNCTION_CALL"
+                        )
+                        & tool_prompt_node.Outputs.results[ToolCallingState.current_prompt_output_index]["value"][
+                            "name"
+                        ].equals(fn_name)
+                    )
+                )
+            )
+
+        for function in functions:
+            if isinstance(function, ComposioToolDefinition):
+                function_name = get_function_name(function)
+                port = create_port_condition(function_name)
+                setattr(Ports, function_name, port)
+            elif isinstance(function, MCPServer):
+                tool_functions: List[MCPToolDefinition] = hydrate_mcp_tool_definitions(function)
+                for tool_function in tool_functions:
+                    name = get_mcp_tool_name(tool_function)
+                    port = create_port_condition(name)
+                    setattr(Ports, name, port)
+            else:
+                function_name = get_function_name(function)
+                port = create_port_condition(function_name)
+                setattr(Ports, function_name, port)
+
+        # Add the else port for when no function conditions match
+        setattr(Ports, "default", Port.on_else())
+    else:
+        # If no functions exist, create a simple Ports class with just a default port
+        Ports = type("Ports", (), {"default": Port(default=True)})
+
+    node = cast(
+        Type[RouterNode],
+        type(
+            "RouterNode",
+            (RouterNode,),
+            {
                 "Ports": Ports,
+                "prompt_outputs": tool_prompt_node.Outputs.results,
                 "__module__": __name__,
             },
         ),
@@ -338,8 +463,8 @@ def create_tool_router_node(
 
 
 def create_function_node(
-    function: Tool,
-    tool_router_node: Type[ToolRouterNode],
+    function: ToolBase,
+    tool_prompt_node: Type[ToolPromptNode],
 ) -> Type[BaseNode]:
     """
     Create a FunctionNode class for a given function.
@@ -349,7 +474,7 @@ def create_function_node(
 
     Args:
         function: The function to create a node for
-        tool_router_node: The tool router node class
+        tool_prompt_node: The tool prompt node class
     """
     if isinstance(function, DeploymentDefinition):
         deployment = function.deployment_id or function.deployment_name
@@ -361,7 +486,7 @@ def create_function_node(
             {
                 "deployment": deployment,
                 "release_tag": release_tag,
-                "function_call_output": tool_router_node.Outputs.results,
+                "function_call_output": tool_prompt_node.Outputs.results,
                 "__module__": __name__,
             },
         )
@@ -374,18 +499,19 @@ def create_function_node(
             (ComposioNode,),
             {
                 "composio_tool": function,
-                "function_call_output": tool_router_node.Outputs.results,
+                "function_call_output": tool_prompt_node.Outputs.results,
                 "__module__": __name__,
             },
         )
         return node
     elif is_workflow_class(function):
+        function.is_dynamic = True
         node = type(
             f"DynamicInlineSubworkflowNode_{function.__name__}",
             (DynamicInlineSubworkflowNode,),
             {
                 "subworkflow": function,
-                "function_call_output": tool_router_node.Outputs.results,
+                "function_call_output": tool_prompt_node.Outputs.results,
                 "__module__": __name__,
             },
         )
@@ -395,8 +521,8 @@ def create_function_node(
             f"FunctionNode_{function.__name__}",
             (FunctionNode,),
             {
-                "function_definition": lambda self, **kwargs: function(**kwargs),
-                "function_call_output": tool_router_node.Outputs.results,
+                "function_definition": lambda self, **kwargs: function(**kwargs),  # ← Revert back to lambda
+                "function_call_output": tool_prompt_node.Outputs.results,
                 "__module__": __name__,
             },
         )
@@ -404,11 +530,57 @@ def create_function_node(
     return node
 
 
-def get_function_name(function: Tool) -> str:
+def create_mcp_tool_node(
+    tool_def: MCPToolDefinition,
+    tool_prompt_node: Type[ToolPromptNode],
+) -> Type[BaseNode]:
+    node = type(
+        f"MCPNode_{tool_def.name}",
+        (MCPNode,),
+        {
+            "mcp_tool": tool_def,
+            "function_call_output": tool_prompt_node.Outputs.results,
+            "__module__": __name__,
+        },
+    )
+    return node
+
+
+def create_else_node(
+    tool_prompt_node: Type[ToolPromptNode],
+) -> Type[ElseNode]:
+    class Ports(ElseNode.Ports):
+        loop_to_router = Port.on_if(
+            ToolCallingState.current_prompt_output_index.less_than(tool_prompt_node.Outputs.results.length())
+        )
+        loop_to_prompt = Port.on_elif(ToolCallingState.current_function_calls_processed.greater_than(0))
+        end = Port.on_else()
+
+    node = cast(
+        Type[ElseNode],
+        type(
+            f"{tool_prompt_node.__name__}_ElseNode",
+            (ElseNode,),
+            {
+                "Ports": Ports,
+                "__module__": __name__,
+            },
+        ),
+    )
+    return node
+
+
+def get_function_name(function: ToolBase) -> str:
     if isinstance(function, DeploymentDefinition):
         name = str(function.deployment_id or function.deployment_name)
         return name.replace("-", "")
     elif isinstance(function, ComposioToolDefinition):
-        return function.name
+        # model post init sets the name to the action if it's not set
+        return function.name  # type: ignore[return-value]
     else:
         return snake_case(function.__name__)
+
+
+def get_mcp_tool_name(tool_def: MCPToolDefinition) -> str:
+    server_name = snake_case(tool_def.server.name)
+    return f"{server_name}__{tool_def.name}"
